(declare (not safe))
;;; This file defines some macros used by the error checking system.

(##define-macro (##every? proc lst)
  `(let ((proc ,proc))
     (let loop ((lst ,lst))
       (or (null? lst)
	   (and (proc (car lst))
		(loop (cdr lst)))))))

;;; Unfortunately, this object is available only with gsc.

(##define-macro (macro-absent-object)
  (##namespace ("c#" absent-object))
  `',absent-object)

(##define-macro (absent-object? obj)
  `(eq? ,obj (macro-absent-object)))

;;; This checks to see that s is a string and ...  Use the source, Luke.

(##define-macro (macro-check-string-start-end s start end form expr)
  (define (flat x) (if (pair? x) (cons (car x) (flat (cdr x))) (list x)))
  `(if (##string? ,s)
       (let ((,end (if (absent-object? ,end) (string-length ,s) ,end)))
	 (if (##fixnum? ,start)
	     (if (##fixnum.>= ,start 0)
		 (if (##fixnum? ,end)
		     (if (and (##fixnum.<= ,start ,end)
			      (##fixnum.<= ,end (string-length ,s)))
			 ,expr
			 ,(if (list? form)
			      `(##raise-range-exception #f ',(car form) ,@(cdr form))
			      `(##raise-range-exception #f #t ',(car form) ,@(flat (cdr form)))))
		     (if (##bignum? ,end)
			 ,(if (list? form)
			      `(##raise-range-exception #f ',(car form) ,@(cdr form))
			      `(##raise-range-exception #f #t ',(car form) ,@(flat (cdr form))))
			 ,(if (list? form)
			      `(##fail-check-exact-integer #f ',(car form) ,@(cdr form))
			      `(##fail-check-exact-integer #f #t ',(car form) ,@(flat (cdr form))))))
		 ,(if (list? form)
		      `(##raise-range-exception #f ',(car form) ,@(cdr form))
		      `(##raise-range-exception #f #t ',(car form) ,@(flat (cdr form)))))
	     (if (##bignum? ,start)
		 ,(if (list? form)
		      `(##raise-range-exception #f ',(car form) ,@(cdr form))
		      `(##raise-range-exception #f #t ',(car form) ,@(flat (cdr form))))
		 ,(if (list? form)
		      `(##fail-check-exact-integer #f ',(car form) ,@(cdr form))
		      `(##fail-check-exact-integer #f #t ',(car form) ,@(flat (cdr form)))))))
       ,(if (list? form)
	    `(##fail-check-string #f ',(car form) ,@(cdr form))
	    `(##fail-check-string #f #t ',(car form) ,@(flat (cdr form))))))

;;; Defines a check-type macro for char-set.

(define-check-type char-set 'char-set char-set?)

;;; Brings max-fixnum to the name space.

(##define-macro (macro-max-fixnum)
   ##max-fixnum)

;;; The range macros always check for fixnums, so this is a fixnum check.

(define-check-index-range-macro fixnum (lambda (x) #t))
(declare (safe))
